/**********************************************************************************************************************************/
/* Project      : TAPER                                                                                                           */
/* Module       : TPAUDIO.H                                                                                                       */
/* Description  : Audio functions (read/write VOC/direct samples)                                                                 */
/* Version type : Module interface                                                                                                */
/* Last changed : 05-02-1999  6:15                                                                                                */
/* Update count : 29                                                                                                              */
/* OS type      : PC (DOS)                                                                                                        */
/* Compiler     : Watcom C32 Optimizing Compiler Version 10.x                                                                     */
/* Linker       : Watcom C Linker                                                                                                 */
/*                                                                                                                                */
/*                Copyleft (C) 1997-2001 ThunderWare Research Center, written by Martijn van der Heide.                           */
/*                                                                                                                                */
/*                This program is free software; you can redistribute it and/or                                                   */
/*                modify it under the terms of the GNU General Public License                                                     */
/*                as published by the Free Software Foundation; either version 2                                                  */
/*                of the License, or (at your option) any later version.                                                          */
/*                                                                                                                                */
/*                This program is distributed in the hope that it will be useful,                                                 */
/*                but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  */
/*                MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   */
/*                GNU General Public License for more details.                                                                    */
/*                                                                                                                                */
/*                You should have received a copy of the GNU General Public License                                               */
/*                along with this program; if not, write to the Free Software                                                     */
/*                Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                                     */
/*                                                                                                                                */
/**********************************************************************************************************************************/

#ifndef TPAUDIO_H_INC
#define TPAUDIO_H_INC

#ifdef EXTERN
#undef EXTERN
#endif
#ifdef __TPAUDIO_MAIN__
#define EXTERN
#else
#define EXTERN extern
#endif

/**********************************************************************************************************************************/
/* Define the Normal Speed timing values                                                                                          */
/**********************************************************************************************************************************/

#define NS_LENPILOT   2168                                                                               /* Length of PILOT pulse */
#define NS_LENSYNC1   667                                                                           /* Length of first SYNC pulse */
#define NS_LENSYNC2   735                                                                            /* Length of last SYNC pulse */
#define NS_LEN0BIT    855                                                                           /* Length of a ZERO bit pulse */
#define NS_LEN1BIT    1710                                                                           /* Length of a ONE bit pulse */
#define NS_HDPILOT    8064                                                                     /* Num pulses in header PILOT tone */
#define NS_DTPILOT    3220                                                                       /* Num pulses in data PILOT tone */
#define NS_SAMPRATE   44100                                                                        /* Best samplerate (44,100 Hz) */
#define NS_LENFIRST   8                                                                            /* Num used bits in first byte */
#define NS_LENLAST    8                                                                             /* Num used bits in last byte */

/**********************************************************************************************************************************/
/* Define the High Speed timing values (ZXS only)                                                                                 */
/**********************************************************************************************************************************/

#define HS_LENSYNC1   334                                                                           /* Length of first SYNC pulse */
#define HS_LENSYNC2   368                                                                            /* Length of last SYNC pulse */
#define HS_LEN0BIT    428                                                                           /* Length of a ZERO bit pulse */
#define HS_LEN1BIT    855                                                                            /* Length of a ONE bit pulse */

/**********************************************************************************************************************************/
/* Define the Turbo Speed timing values (ZXS only)                                                                                */
/**********************************************************************************************************************************/

#define TS_LENSYNC1   283                                                                           /* Length of first SYNC pulse */
#define TS_LENSYNC2   312                                                                            /* Length of last SYNC pulse */
#define TS_LEN0BIT    365                                                                           /* Length of a ZERO bit pulse */
#define TS_LEN1BIT    730                                                                            /* Length of a ONE bit pulse */

/**********************************************************************************************************************************/
/* Define the sample rate for Z80em sample files                                                                                  */
/**********************************************************************************************************************************/

#define Z80EMSAMPRATE  500000

/**********************************************************************************************************************************/
/* Define the decoding schemes: Spectrum                                                                                          */
/**********************************************************************************************************************************/

#define DS_NONE       0                                                                             /* Determine decoding schemes */
#define DS_ROM        1                                                                        /* Force ROM values for the timers */
#define DS_BLEEP      2                                                                              /* Expect Firebird BleepLoad */
#define DS_ALKATRAZ   3                                                                                        /* Expect Alkatraz */
#define DS_SLOCK1     4                                                                                            /* SpeedLock 1 */
#define DS_SLOCK2     5                                                                                            /* SpeedLock 2 */
#define DS_SLOCK3     6                                                                                            /* SpeedLock 3 */
#define DS_SLOCK4     7                                                                                            /* SpeedLock 4 */
#define DS_SLOCK5     8                                                                                            /* SpeedLock 5 */
#define DS_SLOCK6     9                                                                                            /* SpeedLock 6 */
#define DS_SLOCK7     10                                                                                           /* SpeedLock 7 */
#define DS_DIGITAL    11                                                                                   /* Digital Integration */
#define DS_DINAMIC    12                                                                                        /* Dinamic loader */
#define DS_SULOAD     13                                                                        /* Sinclair User covertape loader */
#define DS_YSLOAD     14                                                                         /* Your Sinlair covertape loader */
#define DS_PLAYERS    15                                                                                        /* Players loader */
#define DS_POPS       16                                                                          /* Paul Owens Protection System */
#define DS_MAX        17

/**********************************************************************************************************************************/
/* Define the global variables                                                                                                    */
/**********************************************************************************************************************************/

EXTERN bool        InsertDirectRecording;
EXTERN bool        DecodeDirectRecording;
EXTERN bool        SimulatePlayBlock;
EXTERN bool        InitVOCWriteVars;
EXTERN bool        RealSB_DMAOverrun;

EXTERN struct WindowInfo *__SBWindow;
EXTERN bool        CountPlayTimeLength;
EXTERN double      PlayTimeLength;
EXTERN bool        UsePlayTimeClock;
EXTERN char        PlayTimeString[40];
EXTERN bool        PlayTimeUpdated;
EXTERN bool        SB_DMANoIRQTimeOut;

EXTERN dword       VOCHeaderLocations[1000];
EXTERN word        VOCHeaderLocationIndex;
EXTERN bool        VOCHighSpeedSampleRateAdjust;
EXTERN bool        VOCFrequencyShift;

/**********************************************************************************************************************************/
/* Define the interface functions                                                                                                 */
/**********************************************************************************************************************************/

void  AllocateAudioBuffer          (void);
void  ReleaseAudioBuffer           (void);
void  SetSoftMixerOutput           (void);
bool  SetSoftMixerInput            (void);
bool  SetupVOCReadTimers           (short FileType);
bool  SetupVOCWriteTimers          (short WhichWindow, word *SelectedWriteSampleRate);
bool  DecodeDirectSample           (short WhichWindow, int Fh, dword SampleRate, dword BlockLength, dword *FileIndex,
                                    bool SkipSounContinue, bool *Cancelled, dword XMSIndex);
bool  ReadDirectRecording          (short WhichWindow, int Fh, dword SampleRate, dword BlockLength, dword *FileIndex,
                                    bool SkipSounContinue, bool *Cancelled);
bool  WriteVOCBlock                (short WhichWindow, int Fh, struct TapeEntry_s *DirP, short XMSHandle, dword XMSIndex,
                                    dword *FileIndex, comboent *NextBlock);
bool  FlushWriteBuffer             (bool StopImmediately);
char *GetSB_DMANoIRQTimeOutMessage (void);
void  CleanupTapeBuffers           (void);
bool  Sampler                      (short WhichWindow);

#endif
