; **********************************************************************************************************************************
; * Module       : TPFILTER.ASM                                                                                                    *
; * Executable   : -                                                                                                               *
; * Doc file     : -                                                                                                               *
; * Version type : Library module                                                                                                  *
; * Last changed : 24-06-1996  6:40                                                                                                *
; * Update count : 1                                                                                                               *
; * OS type      : PC (DOS)                                                                                                        *
; * Description  : Graphic User Interface (GUI) Assembler implementation of XMS functions                                          *
; * Assembler    : Microsoft (R) Macro Assembler Version 5.10                                                                      *
; * Linker       : Microsoft (R) QuickC Linker Version 4.10                                                                        *
; * MASM attribs : /mx                                                                                                             *
; * Notes        : Derived from VOC.8 which is part of the source from Z80 v3.05 by Gerton Lunter                                  *
; * Details      :                                                                                                                 *
; *                                                                                                                                *
; *                Copyright 1995-1997 by M. van der Heide                                                                         *
; **********************************************************************************************************************************

bptr        equ     byte ptr                                ; Means we're loading/storing 8 bits
wptr        equ     word ptr                                ; Means we're loading/storing 16 bits
dptr        equ     dword ptr                               ; Means we're loading/storing 32 bits

; 'DOSSEG' gives us support for Microsoft C and Turbo C segment naming and ordering schemes

DOSSEG

; Assemble full '286 code

.286C

; We are in the Large model, so there are multiple code segments. Our default code segment name will be "TPFILTER_TEXT".
; This is acceptable in most cases, except when using the Turbo C integrated development environment.

.MODEL  Large, C
.CODE   TPFILTER_TEXT

; **********************************************************************************************************************************
; * _SpecFilter   Apply a digital filter to a sample stream                                                                        *
; *               Filters out low freq components from buffer starting at `Buffer' to `BufLength', using `TRate' (# of T states    *
; *               per sample) and `BaseLine'.                                                                                      *
; *               We try to filter out things below approx 500 Hz, i.e. period of 7000 T's.                                        *
; *               Averaged_value (N + 1) = alpha_signal (N + 1) + (1 - alpha) averaged_value (N).                                  *
; *               Say N * TRate = 7000, then we want (1 - alpha) ^ N approx 1/2, i.e. 1 - N alpha = 1/2,                           *
; *               i.e. alpha = 1/2 N = TRate / 14000, 256 * alpha = TRate / 55, say TRate / 64.                                    *
; *               In comments, alpha and 1 - alpha denote actually 256 * alpha and 256 * (1 - alpha)                               *
; *               Signal & current average are in range 0-ffff                                                                     *
; *                                                                                                                                *
; * Call:         _SpecFilter (byte far *Buffer, word BufLength, word TRate, word far *BaseLine, byte far *SkipLength)             *
; * Entry:        First time: BaseLine should be 0x8000                                                                            *
; * Return:       None                                                                                                             *
; **********************************************************************************************************************************

                public  _SpecFilter

_SpecFilter     proc    Buffer:ptr, BufLength:word, TRate:word, BaseLine:ptr, SkipLength:ptr

                push    ds                                 ; Save all "important" registers
                push    si
                push    es
                push    di

                mov     ax, [TRate]                        ; 900 T's, approx. 4 kHz
                cmp     ax, 900
                ja      FilterRet                          ; Don't filter with such low sample rates
                add     ax, 64
                mov     cl, 7
                shr     ax, cl
                test    ax, ax
                jnz     Filter2                            ; This is 256 * alpha
                mov     ax, 1
Filter2:        sub     ah, al                             ; ah = 1 - alpha
                xchg    ah, al                             ; Now ah = alpha, al = 1 - alpha
                mov     dx, ax
                mov     si, [BufLength]

                assume ds:nothing, es:nothing              ; Tell MASM that DS and ES do not point to our variables

                lds     bx, dptr Buffer
                les     di, dptr BaseLine
                mov     cx, wptr es:[di]
                les     di, dptr SkipLength

SkipLoop:       push    cx
                mov     cl, bptr es:[di]                   ; Get number of bytes to skip
SkipLoop2:      or      cl, cl                             ; Zero? Then we're done
                jz      SkipEnd
                or      si, si                             ; End of stream? Then we're done
                jz      SkipEnd
                inc     bx
                dec     cl
                dec     si
                jmp     SkipLoop2
SkipEnd:        mov     bptr es:[di], cl                   ; Store # bytes left to skip
                pop     cx

FilterLoop:     or      si, si
                jz      FilterEnd
                push    dx
                xor     ah, ah
                mov     al, dh                             ; alpha
                mov     dh, [bx]                           ; Get byte from buffer
                cmp     dh, 1                              ; VOC Sound Start header?
                jne     NoVOC1
                mov     bptr es:[di], 6                    ; 6 bytes to skip
                pop     dx
                jmp     SkipLoop
NoVOC1:         cmp     dh, 2                              ; VOC Sound Continue header?
                jne     NoVOC2
                mov     bptr es:[di], 4                    ; 4 bytes to skip
                pop     dx
                jmp     SkipLoop
NoVOC2:         cmp     dh, 9                              ; GoldWave Extension header?
                jne     NoVOC3
                mov     bptr es:[di], 16                   ; 16 bytes to skip
                pop     dx
                jmp     SkipLoop
NoVOC3:         mul     dh                                 ; ax = alpha * signal / 256
                sub     dh, 5                              ; Tiny offset
                cmp     dh, ch
                sbb     dh, dh
                mov     [bx], dh                           ; Adjust signal
                xchg    ax, cx                             ; Get current average into ax
                xor     dh, dh
                mul     dx                                 ; axdx = (1 - alpha) * current average
                add     al, al
                adc     cl, ah
                adc     ch, dl                             ; Add axdx to cx, obtaining new curr average
                pop     dx
                inc     bx
                dec     si
                jmp     FilterLoop

FilterEnd:      les     di, dptr BaseLine
                mov     wptr es:[di], cx                      ; Return current baseline value

FilterRet:      pop     di
                pop     es
                pop     si
                pop     ds
                ret 

_SpecFilter     endp

; **********************************************************************************************************************************

END
